package com.canbot.u05.sdk.clientdemo.ImageCaptioning;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.ImageFormat;
import android.hardware.Camera;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.text.TextUtils;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.SurfaceHolder;
import android.widget.Toast;

import com.alibaba.fastjson.JSON;
import com.canbot.u05.IMsgBind;
import com.canbot.u05.IMsgCallBack;
import com.canbot.u05.sdk.clientdemo.ActionId;
import com.canbot.u05.sdk.clientdemo.MsgType;
import com.canbot.u05.sdk.clientdemo.Sensor;
import com.canbot.u05.sdk.clientdemo.StringMsgBean;
import com.canbot.u05.sdk.clientdemo.bean.ClientVersionInfo;
import com.canbot.u05.sdk.clientdemo.bean.WifiStatus;
import com.canbot.u05.sdk.clientdemo.face.BaseActivityNoTitle;
import com.canbot.u05.sdk.clientdemo.util.BitmapUtils;
import com.canbot.u05.sdk.clientdemo.util.Logger;
import com.canbot.u05.sdk.clientdemo.util.PathConstant;
import com.canbot.u05.sdk.clientdemo.util.WifiUtils;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.interjoy.sksmarteyesdk.SKUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by zmp on 2017/7/20.
 */

public abstract class CameraBaseActivity extends BaseActivityNoTitle {

        private String TAG = "CameraBaseActivity";
        private static final int MESSAGE_TYPE_SECOND = 1;
        private static final int MESSAGE_TYPE_FINISH = 2;
        private Camera mCamera;
        private Handler handler;
        private Camera.PreviewCallback cb;
        public String bitmapToBase64(Bitmap bitmap) {
                String result = null;
                ByteArrayOutputStream baos = null;
                try {
                        if (bitmap != null) {
                                baos = new ByteArrayOutputStream();
                                bitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);

                                baos.flush();
                                baos.close();

                                byte[] bitmapBytes = baos.toByteArray();
                                result= Base64Util.encode(bitmapBytes);
//                                result = Base64.encodeToString(bitmapBytes, Base64.DEFAULT);
                        }
                } catch (IOException e) {
                        e.printStackTrace();
                } finally {
                        try {
                                if (baos != null) {
                                        baos.flush();
                                        baos.close();
                                }
                        } catch (IOException e) {
                                e.printStackTrace();
                        }
                }
                return result;
        }

        protected boolean initCamera(SurfaceHolder holder) {
                try {
                        this.mCamera = Camera.open(0);
                        this.mCamera.setPreviewDisplay(holder);
                        return true;
                }
                catch (Exception ioe) {
                        ioe.printStackTrace(System.out);
                        if (mCamera != null) {
                                mCamera.release();
                                mCamera = null;
                        }
                        Logger.e(TAG, "初始化摄像头失败");
                        Toast.makeText(CameraBaseActivity.this, "初始化摄像头失败,请检查摄像头是否安装",Toast.LENGTH_SHORT).show();
                        return false;
                }
        }

        protected void openCamera() {
                if (mCamera != null) {
                        try {
                                Camera.Parameters parameters = mCamera.getParameters();
                                parameters.setPreviewSize(PathConstant.PREVIEW_WIDTH, PathConstant.PREVIEW_HEIGHT);// 设置预览照片的大小
                                parameters.setPictureSize(PathConstant.PREVIEW_WIDTH, PathConstant.PREVIEW_HEIGHT);

                                parameters.setPreviewFrameRate(PathConstant.PREVIEW_RATE);// 设置每秒30帧

                                List<Integer> frameRates = parameters.getSupportedPreviewFrameRates();
                                for (int i : frameRates) {
                                        Logger.d(TAG, "支持的预览帧数:" + i);
                                }
                                // 实现自动对焦
                                SKUtils.printSupportFocusMode(parameters);
                                List<String> focusModes = parameters.getSupportedFocusModes();
                                if (focusModes.contains("continuous-video")) {
                                        parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_VIDEO);
                                }
                                if (focusModes.contains("continuous-picture")) {
                                        parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE);
                                }
                                if (focusModes.contains("torch")) {
                                        parameters.setFocusMode(Camera.Parameters.FLASH_MODE_TORCH);
                                }
                                parameters.setPictureFormat(ImageFormat.JPEG);// 设置照片的格式（只支持JPEG）
                                parameters.setPreviewFormat(ImageFormat.NV21);// (支持NV21、YV12)
                                parameters.setJpegQuality(100);// 设置照片的质量
                                mCamera.setDisplayOrientation(90);
                                mCamera.setParameters(parameters);
                                cb = new Camera.PreviewCallback() {
                                        @Override
                                        public void onPreviewFrame(byte[] data, Camera camera) {
                                                mCamera.setOneShotPreviewCallback(this);
                                                onPreviewCallback(data);
                                        }
                                };
                                mCamera.setOneShotPreviewCallback(cb);
                                mCamera.startPreview();
                                mCamera.cancelAutoFocus();
                                //连续自动对焦
                                mCamera.autoFocus(new Camera.AutoFocusCallback() {
                                        @Override
                                        public void onAutoFocus(boolean success, Camera camera) {
//                                                Logger.e(TAG, "onAutoFocus_success：" + success);
                                                if (success) {
                                                        camera.cancelAutoFocus();

                                                }
                                                mCamera.autoFocus(this);
                                        }
                                });
                                Logger.e(TAG, "初始化摄像头成功");
                        }
                        catch (Exception e) {
                                e.printStackTrace();
                                Logger.e(TAG, "初始化摄像头失败");
                        }
                }
        }

        /**
         * 释放摄像头
         */
        protected synchronized void recycleCamera() {
                Logger.e(TAG, "释放摄像头！recycleCamera");
                if (mCamera != null) {
                        try {
                                mCamera.setPreviewDisplay(null);
                        }
                        catch (IOException e) {
                                e.printStackTrace();
                        }
                        mCamera.setOneShotPreviewCallback(null);
                        mCamera.stopPreview();
                        mCamera.release();
                        mCamera = null;
                }
        }

        /**
         * 摄像头数据预览回调
         *
         * @param data
         */
        protected abstract void onPreviewCallback(byte[] data);

        protected void stopPreview() {
                if (mCamera != null) {
                        mCamera.stopPreview();
                }
        }

        protected void startPreview() {
                if (mCamera != null) {
                        mCamera.startPreview();
                        mCamera.setOneShotPreviewCallback(cb);
                }
        }


        /**
         * 拍照
         *
         * @param callBack
         * @param isPreviewAgain 是否继续拍照
         */
        protected void takePhotoandtell(final ITakePhotoCallBack callBack, final boolean isPreviewAgain) {
                sendToRobot(MsgType.PLAY_TTS, "hello，let me see");
                final String[] image = {null};
                CharSequence faceTime = DateFormat.format("yyyy_MM_dd_HH_mm_ss", System.currentTimeMillis());
                final String fileName = faceTime + "photo.jpg";
                handler = new Handler(){
                        @Override
                        public void handleMessage(Message msg) {
                                super.handleMessage(msg);
                                switch (msg.what){
                                        case MESSAGE_TYPE_SECOND:{
//                                                userInfoTv.setText("Hi,"+String.valueOf(msg.obj));
//                                                final SpeechManager speechManager = (SpeechManager) getUnitManager(FuncConstant.SPEECH_MANAGER);
                                                Log.i("info", "speak: =================== "+String.valueOf(msg.obj));
//                                                speechManager.startSpeak("Welcom back "+String.valueOf(msg.obj));
                                                sendToRobot(MsgType.PLAY_TTS,  String.valueOf(msg.obj));

                                                sendToRobot(MsgType.ACTION, ActionId.SALUTE);
                                                break;
                                        }
                                        case MESSAGE_TYPE_FINISH:{

//                                                final SpeechManager speechManager = (SpeechManager) getUnitManager(FuncConstant.SPEECH_MANAGER);
//                                                speechManager.startSpeak("If you are not sure that your face already exists in the database, Please add face first.");
                                                break;
                                        }
                                }

                        }
                };


                if (mCamera != null) {
                        mCamera.takePicture(null, null, new Camera.PictureCallback() {
                                @Override
                                public void onPictureTaken(byte[] data, Camera camera) {
                                        Bitmap bitmap = BitmapFactory.decodeByteArray(data, 0, data.length);
                                        Bitmap bitmap90 = BitmapUtils.adjustPhotoRotation(bitmap, -90);
                                        BitmapUtils.saveAsFile(bitmap90, fileName, PathConstant.FACES_TEMP);
                                        if (callBack != null) {
                                                callBack.callBack(PathConstant.FACES_ABS_TEMP + fileName);
                                        }
                                        if (isPreviewAgain) {
                                                mCamera.startPreview();
                                                mCamera.setOneShotPreviewCallback(cb);
                                        }
                                        image[0] = bitmapToBase64(bitmap);
                                        new Thread(new Runnable() {
                                                @Override
                                                public void run() {
                                                        try {
                                                                String url = "http://192.168.12.102:5000/api/v1/prediction/nic";
                                                                Map<String, Object> map = new HashMap<>();
                                                                SimpleDateFormat sdf=new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
                                                                Date date=new java.util.Date();
                                                                map.put("image", image[0]);
                                                                String param = GsonUtils.toJson(map);
                                                                Logger.e(TAG, "初始化摄像头成功"+param);
                                                                String result = HttpUtil.postGeneralUrl(url,"application/json",param,"UTF-8");
                                                                Log.i("result", "run:==== "+result);
                                                                Gson gson = new Gson();
                                                                String sentence = null;
                                                                Map<String,Object> user_map =  gson.fromJson(result,new TypeToken<Map<String,Object>>(){}.getType());
                                                                List<Map<String,Object>> user_result = (List<Map<String,Object>>) user_map.get("result");
                                                                Map<String,Object> description = (Map<String,Object>) user_result.get(0);
                                                                for (String string:description.keySet()){
                                                                       sentence= description.get(string).toString();
                                                                }
                                                                Log.i("result2", "run:==== "+ sentence);
//                                                                sendToRobot(MsgType.PLAY_TTS, sentence + "&&recoder");
                                                                Message message = Message.obtain();
                                                                message.obj = sentence;
                                                                handler.sendMessage(message);
                                                                message.what = MESSAGE_TYPE_SECOND;


                                                        } catch (Exception e) {
                                                                e.printStackTrace();
                                                        }



                                                }
                                        }).start();
                                        deleteMoreFiles(fileName);
                                }
                        });
                }

        }


        private void deleteMoreFiles(final String fileName) {
                File file = new File(PathConstant.FACES_TEMP);
                if (file.exists() && file.isDirectory()) {
                        File[] files = file.listFiles(new FileFilter() {
                                @Override
                                public boolean accept(File file) {
                                        return !file.getAbsolutePath().contains(fileName);
                                }
                        });
                        if (files != null && files.length > 0) {
                                Arrays.sort(files);
                                if (files.length > 10) {
                                        for (int i = 0; i < files.length - 5; i++) {
                                                files[i].delete();
                                        }
                                }
                        }
                }
        }

        protected interface ITakePhotoCallBack {

                void callBack(String photoName);
        }

        @Override
        protected void onResume() {
                super.onResume();
                bindService();
        }

        @Override
        protected void onPause() {
                unRegisterAIDL();
                unbindService(connection);
                super.onPause();
        }
        //---------------------------------------------------------------------


        //=====================================================================
        public void sendToRobot(int msgType, String data) {
                StringMsgBean msgBean = new StringMsgBean();
                msgBean.setMsgType(msgType);
                msgBean.setMsgData(data);
                sendData(msgBean);
        }

        private boolean handleResult(int msgType, String msgData) {
                switch (msgType) {
                        case MsgType.ACTION_EXE_FINISH:
                                Log.e(TAG, "handleResult: action finish ,action id  = "+msgData );
                                break;
                        case MsgType.RECEIVER_MSG_VOICE_RECOGNIZER_RESULT:
                                if (msgData.equals("今天天气怎么样")) {     //本地处理
                                        Log.e(TAG, "收到识别结果，本地需要处理: " + msgData);
                                        sendToRobot(MsgType.PLAY_TTS, "今天天气不错" + "&&recoder");
                                        return true;

                                }
                                else {    //本地不处理
                                        Log.e(TAG, "收到识别结果,本地不处理: " + msgData);
                                        return false;

                                }
                        case MsgType.RECEIVER_MSG_SENSOR:   //感应器信息
                                int which = Integer.parseInt(msgData);
                                if (which == Sensor.Belly) {
                                        Log.e(TAG, "肚子感应器被触摸: ");
                                }
                                else if (which == Sensor.Buns) {
                                        Log.e(TAG, "屁股感应器被触摸: ");
                                }
                                else if (which == Sensor.Right_ear) {
                                        Log.e(TAG, "右耳感应器被触摸: ");
                                }
                                else if (which == Sensor.Mouth) {
                                        Log.e(TAG, "下巴感应器被触摸: ");
                                }
                                else if (which == Sensor.Head_top) {
                                        Log.e(TAG, "头顶感应器被触摸: ");
                                }
                                else if (which == Sensor.Left_ear) {
                                        Log.e(TAG, "左耳感应器被触摸: ");
                                }
                                else if (which == Sensor.HUMAN_BODY_FRONT) {
                                        Log.e(TAG, "人体感应器(前面)被触发: ");
                                }
                                else if (which == Sensor.HUMAN_BODY_BACK) {
                                        Log.e(TAG, "人体感应器(后面)被触发: ");
                                }
                                return true;
                        case MsgType.RECEIVER_MSG_WAKE_UP:   //头部唤醒信息
                                Log.e(TAG, "收到头部唤醒: " + msgData);
                                sendToRobot(MsgType.PLAY_TTS, "我在呢" + "&&recoder");
                                sendToRobot(MsgType.RECEIVER_WAKEUP_SOURCE_ROTATE, msgData);
                                return true;
                        case MsgType.RECEIVER_MSG_PLAY_SOUND_END: //语音播放结束
                                Log.e(TAG, "语音播放结束: " + msgData);
                                return true;
                        case MsgType.RECEIVER_MSG_SUSPEND:  //机器人进入休眠（待唤醒）状态
                                Log.e(TAG, "收到机器人进入休眠状态: " + msgData);
                                return true;
                        case MsgType.RECEIVE_POWER_INFO:
                                Log.e(TAG, "收到机器人电量信息: " + msgData); //收到电量信息（需要先发送查询电量指令）
                                return true;
                        case MsgType.RECEIVER_MSG_WIFI_STATUS_SSID:
                                Log.e(TAG, "收到机器人头部WIFI状态信息: " + msgData); //1.若头部网路状态改变（如网络断开），则会主动发送消息指令;2.客户端可主动发送发送查询wifi状态 指令“SEND_MSG_WIFI_STATUS_SSID”。
                                String[] strings = msgData.split(",");
                                WifiStatus wifiStatus = new WifiStatus(strings[0], strings[1]);
                                Log.e(TAG, "收到机器人头部WIFI状态信息: " + wifiStatus.toString());
                                return true;
                        case MsgType.U05_ROBOT_WIFI_SSID: //收到机器人自身热点SSID（需要先发送查询指令 QUERY_U05_ROBOT_WIFI_SSID , 机器人热点密码是1223334444）
                                if (!TextUtils.isEmpty(msgData)) {
                                        Log.e(TAG, "收到机器人自身热点ssid: " + msgData);
                                }
                                return true;
                        case MsgType.HEAD_NO_WIFI_INFO://头部没有wifi信息时的回调
                                runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                                Toast.makeText(CameraBaseActivity.this, "头部没有连接WIFI", Toast.LENGTH_SHORT).show();
                                        }
                                });
                                Log.e(TAG, "头部没有WIFI");
                                break;
                        case MsgType.RECEIVE_MSG_WIFI_SSID_AND_PWD://接收到发送当前连接的 wifi ssid 及 password
                                if (msgData.contains(":")) {
                                        String[] datas = msgData.split(":");
                                        final String ssid = datas[0];
                                        if (datas.length > 1) {
                                                final String pwd = datas[1];
                                                runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                                Toast.makeText(CameraBaseActivity.this, "头部WIFI,ssid是:" + ssid + "pwd是:" + pwd, Toast.LENGTH_SHORT).show();
                                                        }
                                                });
                                                //胸口连接网络
                                                WifiUtils.getInstance().connect(ssid, pwd);

                                        }
                                        else {
                                                Logger.e(TAG, "头部发送的 wifi 不正确");
                                        }
                                }
                                return true;
//                        case MsgType.VOLUME:
//                                if (!TextUtils.isEmpty(msgData)) {
//                                        String s[] = msgData.split("#");
//                                        if (s != null && s.length == 2) {
//                                                if (!TextUtils.isEmpty(s[0]) && !TextUtils.isEmpty(s[1])) {
//                                                        if (s[1].equals("+")) {
//                                                                msgData = s[0] + "#" + "声音已经调大了";
//                                                        }
//                                                        if (s[1].equals("-")) {
//                                                                msgData = s[0] + "#" + "/ai/ai05res/a1/res/audio/dance/self_introduction_my.wav";
//                                                        }
//                                                        if (s[1].equals("max")) {
//                                                                msgData = s[0] + "#" + "声音已经最大了";
//                                                        }
//                                                        if (s[1].equals("min")) {
//                                                                msgData = s[0] + "#" + "声音已经最小了";
//                                                        }
//
//                                                }
//                                        }
//                                }
//                                Logger.e(TAG, msgData + "---------");
//                                sendToRobot(MsgType.VOLUME, msgData);
//                                return true;
                        case MsgType.RECEIVER_CHARGE_STATUS://接受充电状态
                                switch (msgData) {
                                        case "0":
                                                runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                                Toast.makeText(CameraBaseActivity.this, "没有充电", Toast.LENGTH_SHORT).show();
                                                        }
                                                });
                                                break;
                                        case "1":
                                                runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                                Toast.makeText(CameraBaseActivity.this, "正在充电", Toast.LENGTH_SHORT).show();
                                                        }
                                                });
                                                break;
                                        case "2":
                                                runOnUiThread(new Runnable() {
                                                        @Override
                                                        public void run() {
                                                                Toast.makeText(CameraBaseActivity.this, "正在自动回冲", Toast.LENGTH_SHORT).show();
                                                        }
                                                });
                                                break;
                                        default:
                                                break;
                                }
                                return true;
                        case MsgType.SEND_CLIENT_VERSION: {//接受版本号
                                if (TextUtils.isEmpty(msgData)) {
                                        runOnUiThread(new Runnable() {
                                                @Override
                                                public void run() {
                                                        Toast.makeText(CameraBaseActivity.this, "查询超时", Toast.LENGTH_SHORT).show();
                                                }
                                        });
                                } else {
                                        ClientVersionInfo versionInfo = JSON.parseObject(msgData, ClientVersionInfo.class);
                                        Log.d(TAG, "versionInfo:" + versionInfo);
                                }
                                break;
                        }

                }
                return false;
        }
        protected void bindService() {
                try {
                        Intent intent = new Intent("com.canbot.u05.service.MsgService");
                        intent.setClassName("com.canbot.u05", "com.canbot.u05.service.MsgService");
                        bindService(intent, connection, Context.BIND_AUTO_CREATE);
                } catch (Exception e) {
                        Log.e(TAG, "bindService-----Exception--" + e.toString());
                        e.printStackTrace();
                }
        }

        /**
         * 解绑远程服务
         */
        protected void unBindMsgService() {
                unbindService(connection);
        }

        /**
         * 绑定远程接口
         */
        private void registerAIDL() {
                if (mService != null) {
                        try {
                                mService.registerCallBack(iMsgCallback);
                                Log.e(TAG, "registerAIDL---------");
                        } catch (RemoteException e) {
                                Log.e(TAG, "RemoteException-------" + e.toString());
                        }
                }
        }

        /**
         * 解绑远程接口
         */
        protected void unRegisterAIDL() {
                if (mService != null) {
                        try {
                                mService.unRegisterCallBack(iMsgCallback);
                                mService = null;
                                Log.e(TAG, "unRegisterAIDL---------");
                        } catch (RemoteException e) {
                                e.printStackTrace();
                        }
                }
        }
        private IMsgBind mService;

        protected ServiceConnection connection = new ServiceConnection() {
                @Override
                public void onServiceDisconnected(ComponentName name) {
                        Log.e(TAG, "onServiceDisconnected-------");

                }

                @Override
                public void onServiceConnected(ComponentName name, IBinder service) {
                        Log.e(TAG, "onServiceConnected-------");
                        mService = IMsgBind.Stub.asInterface(service);
                        registerAIDL();
                }
        };
        private IMsgCallBack iMsgCallback = new IMsgCallBack.Stub() {
                @Override
                public boolean interceptRobotAction(final String dataJson) {
                        Log.e(TAG, "onReceiver-------data=" + dataJson);
                        StringMsgBean stringMsgBean = JSON.parseObject(dataJson, StringMsgBean.class);
                        final int msgType = stringMsgBean.getMsgType();
                        final String msgData = stringMsgBean.getMsgData();
                        return handleResult(msgType, msgData);
                }
        };

        /**
         * 通过aidl发送数据
         *
         * @param msgBean
         */
        protected void sendData(StringMsgBean msgBean) {
                if (mService != null) {
                        String jsonString = JSON.toJSONString(msgBean);
                        try {
                                Log.e(TAG, "sendData=" + jsonString);
                                mService.send(jsonString);
                        } catch (RemoteException e) {
                                e.printStackTrace();
                                Log.e(TAG, "RemoteException=" + e.toString());
                        }
                }
        }
}
